@@ -4,20 +4,15 @@
  * For license terms, see the file COPYING in this directory.
  */
 #include "config.h"
+#include "fetchmail.h"
 
 #include <stdio.h>
-#if defined(STDC_HEADERS)
 #include <stdlib.h>
-#endif
-#if defined(HAVE_UNISTD_H)
 #include <unistd.h>
-#endif
 #include <fcntl.h>
 #include <string.h>
 #include <signal.h>
-#if defined(HAVE_SYSLOG)
 #include <syslog.h>
-#endif
 #include <pwd.h>
 #ifdef __FreeBSD__
 #include <grp.h>
@@ -25,25 +20,21 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_SETRLIMIT
 #include <sys/resource.h>
-#endif /* HAVE_SETRLIMIT */
 
 #ifdef HAVE_SOCKS
 #include <socks.h> /* SOCKSinit() */
 #endif /* HAVE_SOCKS */
 
-#ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
-#endif
 
-#include "fetchmail.h"
 #include "socket.h"
 #include "tunable.h"
 #include "smtp.h"
 #include "netrc.h"
 #include "i18n.h"
 #include "lock.h"
+#include "getopt.h" // for optind
 
 /* need these (and sys/types.h) for res_init() */
 #include <netinet/in.h>
@@ -89,10 +80,11 @@ static int querystatus;		/* status of query */
 static int successes;		/* count number of successful polls */
 static int activecount;		/* count number of active entries */
 static struct runctl cmd_run;	/* global options set from command line */
-static time_t parsetime;	/* time of last parse */
+static time_t parsetime;	/* time of last parse of fetchmail's rcfile */
+static time_t netrc_parsetime;  /* time of last parse of .netrc file */
 
-static RETSIGTYPE terminate_run(int);
-static RETSIGTYPE terminate_poll(int);
+static void terminate_run(int);
+static void terminate_poll(int);
 
 #if defined(__FreeBSD__) && defined(__FreeBSD_USE_KVM)
 /* drop SGID kmem privileage until we need it */
@@ -115,7 +107,7 @@ static void dropprivs(void)
 }
 #endif
 
-#if defined(HAVE_SETLOCALE) && defined(ENABLE_NLS) && defined(HAVE_STRFTIME)
+#if defined(ENABLE_NLS)
 #include <locale.h>
 /** returns timestamp in current locale,
  * and resets LC_TIME locale to POSIX. */
@@ -134,7 +126,7 @@ static char *timestamp (void)
 #define timestamp rfc822timestamp
 #endif
 
-static RETSIGTYPE donothing(int sig) 
+static void donothing(int sig) 
 {
     set_signal_handler(sig, donothing);
     lastsig = sig;
@@ -156,6 +148,49 @@ static void printcopyright(FILE *fp) {
 #endif
 }
 
+static void reportquerystatus(FILE *fp) {
+	const char *desc;
+	switch(querystatus)
+	{
+	case PS_SUCCESS:
+	    desc = "SUCCESS"; break;
+	case PS_NOMAIL:
+	    desc = "NOMAIL"; break;
+	case PS_SOCKET:
+	    desc = "SOCKET"; break;
+	case PS_AUTHFAIL:
+	    desc = "AUTHFAIL"; break;
+	case PS_PROTOCOL:
+	    desc = "PROTOCOL"; break;
+	case PS_SYNTAX:
+	    desc = "SYNTAX"; break;
+	case PS_IOERR:
+	    desc = "IOERR"; break;
+	case PS_ERROR:
+	    desc = "ERROR"; break;
+	case PS_EXCLUDE:
+	    desc = "EXCLUDE"; break;
+	case PS_LOCKBUSY:
+	    desc = "LOCKBUSY"; break;
+	case PS_SMTP:
+	    desc = "SMTP"; break;
+	case PS_DNS:
+	    desc = "DNS"; break;
+	case PS_BSMTP:
+	    desc = "BSMTP"; break;
+	case PS_MAXFETCH:
+	    desc = "MAXFETCH"; break;
+	default:
+	    desc = NULL;
+	}
+
+	if (desc) {
+	    report(fp, GT_("Query status=%d (%s)\n"), querystatus, desc);
+	} else {
+	    report(fp, GT_("Query status=%d\n"), querystatus);
+	}
+}
+
 const char *iana_charset;
 
 int main(int argc, char **argv)
@@ -165,8 +200,8 @@ int main(int argc, char **argv)
     flag safewithbg = FALSE; /** if parsed options are compatible with a
 			      fetchmail copy running in the background */
     struct query *ctl;
-    netrc_entry *netrc_list;
-    char *netrc_file, *tmpbuf;
+    char *netrc_file; /** long-lived variable containing the name of the .netrc path */
+    char *tmpbuf;
     pid_t pid;
     int lastsig = 0;
 
@@ -268,22 +303,16 @@ int main(int argc, char **argv)
 	"-ODMR"
 #endif /* ODMR_ENABLE */
 #ifndef SSL_ENABLE
-	"-SSL"
+	"-TLS"
 #else
-#ifdef USING_WOLFSSL
+# ifdef USING_WOLFSSL
 	"+WOLFSSL"
-#else
-	"+SSL-SSLv2"
-#endif
-#if (HAVE_DECL_SSLV3_CLIENT_METHOD + 0 == 0) || defined(OPENSSL_NO_SSL3)
-	"-SSLv3"
-#endif
-#if HAVE_DECL_TLS1_2_VERSION + 0 == 0
-	"-TLS1.2"
-#endif
-#if HAVE_DECL_TLS1_3_VERSION + 0 == 0
+# else
+	"+TLS"
+# endif
+# if HAVE_DECL_TLS1_3_VERSION + 0 == 0
 	"-TLS1.3"
-#endif
+# endif
 #endif
 #ifdef OPIE_ENABLE
 	"+OPIE"
@@ -318,7 +347,7 @@ int main(int argc, char **argv)
 			OpenSSL_version(OPENSSL_DIR),
 			OpenSSL_version(OPENSSL_ENGINES_DIR));
 # if !HAVE_DECL_TLS1_3_VERSION || defined(OPENSSL_NO_TLS1_3)
-	printf(GT_("WARNING: Your SSL/TLS library does not support TLS v1.3.\n"));
+# error Your SSL/TLS library does not support TLS v1.3.
 # endif
 # if defined(LIBRESSL_VERSION_NUMBER) && !defined(__OpenBSD__)
 	/* OpenBSD ships LibreSSL as part of the base system, so is exempt
@@ -332,13 +361,6 @@ int main(int argc, char **argv)
 	puts("");
 	printcopyright(stdout);
 	puts("");
-	fputs("Fallback MDA: ", stdout);
-#ifdef FALLBACK_MDA
-	fputs(FALLBACK_MDA, stdout);
-#else
-	fputs("(none)", stdout);
-#endif
-	putchar('\n');
 	fflush(stdout);
 
 	/* this is an attempt to help remote debugging */
@@ -379,16 +401,10 @@ int main(int argc, char **argv)
 	}
     }
 
-#if defined(HAVE_SYSLOG)
     /* logging should be set up early in case we were restarted from exec */
     if (run.use_syslog)
     {
-#if defined(LOG_MAIL)
 	openlog(program_name, LOG_PID, LOG_MAIL);
-#else
-	/* Assume BSD4.2 openlog with two arguments */
-	openlog(program_name, LOG_PID);
-#endif
 	/* precedence: logfile (if effective) overrides syslog. */
 	if (run.logfile) {
 	    syslog(LOG_ERR, GT_("syslog and logfile options are both set, ignoring syslog, and logging to %s"), run.logfile);
@@ -399,7 +415,6 @@ int main(int argc, char **argv)
 	}
     }
     else
-#endif
 	report_init((run.poll_interval == 0 || nodetach) && !run.logfile); /* when changing this, change copy above, too */
 
 #ifdef POP3_ENABLE
@@ -420,7 +435,6 @@ int main(int argc, char **argv)
     /* construct the lockfile */
     fm_lock_setup(&run);
 
-#ifdef HAVE_SETRLIMIT
     /*
      * Before getting passwords, disable core dumps unless -v -d0 mode is on.
      * Core dumps could otherwise contain passwords to be scavenged by a
@@ -433,14 +447,20 @@ int main(int argc, char **argv)
 	corelimit.rlim_max = 0;
 	setrlimit(RLIMIT_CORE, &corelimit);
     }
-#endif /* HAVE_SETRLIMIT */
 
-#define	NETRC_FILE	".netrc"
     /* parse the ~/.netrc file (if present) for future password lookups. */
     netrc_file = prependdir (NETRC_FILE, home);
+    netrc_entry *netrc_list;
     netrc_list = parse_netrc(netrc_file);
-    free(netrc_file);
-#undef NETRC_FILE
+    netrc_parsetime = time(NULL);
+    { // do not put this under netrc_list - if .netrc were empty, netrc_list would be NULL.
+        // note the parse time of netrc
+        struct stat st;
+        if (0 == stat(netrc_file, &st)) {
+                netrc_parsetime = st.st_mtime;
+        }
+    }
+    /* do NOT free(netrc_file) -- we need it later in the main loop for checking if its st_mtime changed */
 
     /* pick up passwords where we can */ 
     for (ctl = querylist; ctl; ctl = ctl->next)
@@ -453,15 +473,16 @@ int main(int argc, char **argv)
 		ctl->password = ctl->remotename;
 	    else
 	    {
-		netrc_entry *p;
+		const netrc_entry *p;
 
 		/* look up the pollname and account in the .netrc file. */
 		p = search_netrc(netrc_list,
 				 ctl->server.pollname, ctl->remotename);
 		/* if we find a matching entry with a password, use it */
 		if (p && p->password)
+		{
 		    ctl->password = xstrdup(p->password);
-
+		}
 		/* otherwise try with "via" name if there is one */
 		else if (ctl->server.via)
 		{
@@ -472,10 +493,16 @@ int main(int argc, char **argv)
 		}
 	    }
 	}
+	if (ctl->moveto != NULL && ctl->server.protocol != P_IMAP)
+	{
+	    (void) fprintf(stderr,
+			   GT_("fetchmail: configuration invalid, --moveto is only valid for IMAP servers\n"));
+	    exit(PS_SYNTAX);
+	}
     }
 
     free_netrc(netrc_list);
-    netrc_list = 0;
+    netrc_list = NULL;
 
     /* perhaps we just want to check options? */
     if (versioninfo)
@@ -556,59 +583,63 @@ int main(int argc, char **argv)
     /* another fetchmail is running -- wake it up or die */
     if (pid != 0)
     {
-	if (check_only)
-	{
-	    fprintf(stderr,
-		 GT_("fetchmail: can't check mail while another fetchmail to same host is running.\n"));
-	    return(PS_EXCLUDE);
-        }
-	else if (!implicitmode)
-	{
-	    fprintf(stderr,
-		 GT_("fetchmail: can't poll specified hosts with another fetchmail running at %ld.\n"),
-		 (long)pid);
-		return(PS_EXCLUDE);
-	}
-	else if (!bkgd)
-	{
-	    fprintf(stderr,
-		 GT_("fetchmail: another foreground fetchmail is running at %ld.\n"),
-		 (long)pid);
-		return(PS_EXCLUDE);
-	}
-	else if (getpid() == pid)
+	if (getpid() == pid) {
 	    /* this test enables re-execing on a changed rcfile */
 	    fm_lock_assert();
-	else if (argc > 1 && !safewithbg)
-	{
-	    fprintf(stderr,
-		    GT_("fetchmail: can't accept options while a background fetchmail is running.\n"));
+	} else {
+	    xfree(netrc_file);
+	    if (check_only)
 	    {
-		int i;
-		fprintf(stderr, "argc = %d, arg list:\n", argc);
-		for (i = 1; i < argc; i++) fprintf(stderr, "arg %d = \"%s\"\n", i, argv[i]);
-	    }
-	    return(PS_EXCLUDE);
-	}
-	else if (kill(pid, SIGUSR1) == 0)
-	{
-	    if (outlevel > O_SILENT)
 		fprintf(stderr,
-			GT_("fetchmail: background fetchmail at %ld awakened.\n"),
+			GT_("fetchmail: can't check mail while another fetchmail to same host is running.\n"));
+		return (PS_EXCLUDE);
+	    }
+	    else if (!implicitmode)
+	    {
+		fprintf(stderr,
+			GT_("fetchmail: can't poll specified hosts with another fetchmail running at %ld.\n"),
 			(long)pid);
-	    return(0);
-	}
-	else
-	{
-	    /*
-	     * Should never happen -- possible only if a background fetchmail
-	     * croaks after the first kill probe above but before the
-	     * SIGUSR1/SIGHUP transmission.
-	     */
-	    fprintf(stderr,
-		    GT_("fetchmail: elder sibling at %ld died mysteriously.\n"),
-		    (long)pid);
-	    return(PS_UNDEFINED);
+		return (PS_EXCLUDE);
+	    }
+	    else if (!bkgd)
+	    {
+		fprintf(stderr,
+			GT_("fetchmail: another foreground fetchmail is running at %ld.\n"),
+			(long)pid);
+		return (PS_EXCLUDE);
+	    }
+	    else if (argc > 1 && !safewithbg)
+	    {
+		fprintf(stderr,
+			GT_("fetchmail: can't accept options while a background fetchmail is running.\n"));
+		{
+		    int i;
+		    fprintf(stderr, "argc = %d, arg list:\n", argc);
+		    for (i = 1; i < argc; i++)
+			fprintf(stderr, "arg %d = \"%s\"\n", i, argv[i]);
+		}
+		return (PS_EXCLUDE);
+	    }
+	    else if (kill(pid, SIGUSR1) == 0)
+	    {
+		if (outlevel > O_SILENT)
+		    fprintf(stderr,
+			    GT_("fetchmail: background fetchmail at %ld awakened.\n"),
+			    (long)pid);
+		return (0);
+	    }
+	    else
+	    {
+		/*
+		 * Should never happen -- possible only if a background fetchmail
+		 * croaks after the first kill probe above but before the
+		 * SIGUSR1/SIGHUP transmission.
+		 */
+		fprintf(stderr,
+			GT_("fetchmail: elder sibling at %ld died mysteriously.\n"),
+			(long)pid);
+		return (PS_UNDEFINED);
+	    }
 	}
     }
 
@@ -623,6 +654,7 @@ int main(int argc, char **argv)
 		fprintf(stderr,
 			GT_("fetchmail: can't find a password for %s@%s.\n"),
 			ctl->remotename, ctl->server.pollname);
+		xfree(netrc_file);
 		return(PS_AUTHFAIL);
 	    } else {
 		const char* password_prompt = GT_("Enter password for %s@%s: ");
@@ -724,6 +756,9 @@ int main(int argc, char **argv)
 	 * leaks...
 	 */
 	struct stat	rcstat;
+        struct stat	netrcstat;
+        memset(&rcstat, 0, sizeof rcstat);
+        memset(&netrcstat, 0, sizeof netrcstat);
 
 	if (strcmp(rcfile, "-") == 0) {
 	    /* do nothing */
@@ -733,9 +768,14 @@ int main(int argc, char **argv)
 		       GT_("couldn't time-check %s (error %d)\n"),
 		       rcfile, errno);
 	}
-	else if (rcstat.st_mtime > parsetime)
+        if (-1 == stat(netrc_file, &netrcstat) && ENOENT != errno) {
+                report(stderr, 
+                        GT_("couldn't time-check %s (error %d)\n"),
+                        netrc_file, errno);
+        }
+	else if (rcstat.st_mtime > parsetime || netrcstat.st_mtime > netrc_parsetime)
 	{
-	    report(stdout, GT_("restarting fetchmail (%s changed)\n"), rcfile);
+	    report(stdout, GT_("restarting fetchmail (%s changed)\n"), rcstat.st_mtime > parsetime ? rcfile : netrc_file);
 
 	    /* restore the startup directory */
 	    if (!currentwd[0] || chdir (currentwd) == -1)
@@ -764,6 +804,7 @@ int main(int argc, char **argv)
 	     * path wasn't used in the initial call.  (If I recall
 	     * correctly, Linux saves it but many other Unices don't.)
 	     */
+            xfree(netrc_file);
 	    execvp(argv[0], argv);
 	    report(stderr, GT_("attempt to re-exec fetchmail failed\n"));
 	}
@@ -842,40 +883,7 @@ int main(int argc, char **argv)
 			successes++;
 		    else if (!check_only && 
 			     ((querystatus!=PS_NOMAIL) || (outlevel==O_DEBUG)))
-			switch(querystatus)
-			{
-			case PS_SUCCESS:
-			    report(stdout,GT_("Query status=0 (SUCCESS)\n"));break;
-			case PS_NOMAIL: 
-			    report(stdout,GT_("Query status=1 (NOMAIL)\n")); break;
-			case PS_SOCKET:
-			    report(stdout,GT_("Query status=2 (SOCKET)\n")); break;
-			case PS_AUTHFAIL:
-			    report(stdout,GT_("Query status=3 (AUTHFAIL)\n"));break;
-			case PS_PROTOCOL:
-			    report(stdout,GT_("Query status=4 (PROTOCOL)\n"));break;
-			case PS_SYNTAX:
-			    report(stdout,GT_("Query status=5 (SYNTAX)\n")); break;
-			case PS_IOERR:
-			    report(stdout,GT_("Query status=6 (IOERR)\n"));  break;
-			case PS_ERROR:
-			    report(stdout,GT_("Query status=7 (ERROR)\n"));  break;
-			case PS_EXCLUDE:
-			    report(stdout,GT_("Query status=8 (EXCLUDE)\n")); break;
-			case PS_LOCKBUSY:
-			    report(stdout,GT_("Query status=9 (LOCKBUSY)\n"));break;
-			case PS_SMTP:
-			    report(stdout,GT_("Query status=10 (SMTP)\n")); break;
-			case PS_DNS:
-			    report(stdout,GT_("Query status=11 (DNS)\n")); break;
-			case PS_BSMTP:
-			    report(stdout,GT_("Query status=12 (BSMTP)\n")); break;
-			case PS_MAXFETCH:
-			    report(stdout,GT_("Query status=13 (MAXFETCH)\n"));break;
-			default:
-			    report(stdout,GT_("Query status=%d\n"),querystatus);
-			    break;
-			}
+			reportquerystatus(stdout);
 
 #ifdef CAN_MONITOR
 		    if (ctl->server.monitor)
@@ -945,12 +953,12 @@ int main(int argc, char **argv)
 	    if ((lastsig = interruptible_idle(run.poll_interval)))
 	    {
 		if (outlevel > O_SILENT)
-#ifdef SYS_SIGLIST_DECLARED
-		    report(stdout, 
-		       GT_("awakened by %s\n"), sys_siglist[lastsig]);
+#ifdef HAVE_STRSIGNAL
+		    report(stdout, GT_("awakened by %s\n"), strsignal(lastsig));
+#elif SYS_SIGLIST_DECLARED
+		    report(stdout, GT_("awakened by %s\n"), sys_siglist[lastsig]);
 #else
-	    	    report(stdout, 
-		       GT_("awakened by signal %d\n"), lastsig);
+		    report(stdout, GT_("awakened by signal %d\n"), lastsig);
 #endif
 		for (ctl = querylist; ctl; ctl = ctl->next)
 		    ctl->wedged = FALSE;
@@ -968,6 +976,8 @@ int main(int argc, char **argv)
 
     terminate_run(0);
 
+    xfree(netrc_file);
+
     if (successes)
 	exit(PS_SUCCESS);
     else if (querystatus)
@@ -1010,6 +1020,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(server.interval);
     FLAG_MERGE(server.authenticate);
     FLAG_MERGE(server.timeout);
+    FLAG_MERGE(server.idle_timeout);
     STRING_MERGE(server.envelope);
     FLAG_MERGE(server.envskip);
     STRING_MERGE(server.qvirtual);
@@ -1044,6 +1055,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     STRING_MERGE(postconnect);
 
     FLAG_MERGE(keep);
+    STRING_MERGE(moveto);
     FLAG_MERGE(flush);
     FLAG_MERGE(limitflush);
     FLAG_MERGE(fetchall);
@@ -1055,6 +1067,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(dropdelivered);
     FLAG_MERGE(mimedecode);
     FLAG_MERGE(idle);
+    FLAG_MERGE(forceidle);
     FLAG_MERGE(limit);
     FLAG_MERGE(warnings);
     FLAG_MERGE(fetchlimit);
@@ -1101,6 +1114,7 @@ static int load_params(int argc, char **argv, int optind)
     def_opts.smtpname = (char *)0;
     def_opts.server.protocol = P_AUTO;
     def_opts.server.timeout = CLIENT_TIMEOUT;
+    def_opts.server.idle_timeout = CLIENT_IDLE_TIMEOUT;
     def_opts.server.esmtp_name = user;
     def_opts.server.badheader = BHREJECT;
     def_opts.warnings = WARNING_INTERVAL;
@@ -1131,14 +1145,15 @@ static int load_params(int argc, char **argv, int optind)
     }
 
     /* this builds the host list */
-    if ((st = prc_parse_file(rcfile, !versioninfo)) != 0)
+    if ((st = prc_parse_file(rcfile, !versioninfo)) != 0) {
 	/*
 	 * FIXME: someday, send notification mail here if backgrounded.
 	 * Right now, that can happen if the user changes the rcfile
 	 * while the fetchmail is running in background.  Do similarly
 	 * for the other exit() calls in this function.
 	 */
-	exit(st);
+	// exit(st); // -- deferred, see below
+    }
 
     if ((implicitmode = (optind >= argc)))
     {
@@ -1318,6 +1333,7 @@ static int load_params(int argc, char **argv, int optind)
 	    DEFAULT(ctl->dropdelivered, FALSE);
 	    DEFAULT(ctl->mimedecode, FALSE);
 	    DEFAULT(ctl->idle, FALSE);
+	    DEFAULT(ctl->forceidle, FALSE);
 	    DEFAULT(ctl->server.dns, TRUE);
 	    DEFAULT(ctl->server.uidl, FALSE);
 	    DEFAULT(ctl->use_ssl, FALSE);
@@ -1419,15 +1435,17 @@ static int load_params(int argc, char **argv, int optind)
 				   ctl->server.pollname);
 		    exit(PS_SYNTAX);
 		}
-		switch (ctl->server.protocol) {
+		if (!ctl->server.plugin) { // do not check this under plugin, we don't know what protocol the plugin talks, Debian Bug#1076604
+		    switch (ctl->server.protocol) {
 			case P_POP3: case P_APOP:
-				if (port == 995 && !ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need --ssl for port 995/service pop3s.\n"), ctl->server.pollname);
-				if (port == 110 &&  ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need port 995/service pop3s for --ssl.\n"), ctl->server.pollname);
-				break;
+			    if (port == 995 && !ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need --ssl for port 995/service pop3s.\n"), ctl->server.pollname);
+			    if (port == 110 &&  ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need port 995/service pop3s for --ssl.\n"), ctl->server.pollname);
+			    break;
 			case P_IMAP:
-				if (port == 993 && !ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need --ssl for port 993/service imaps.\n"), ctl->server.pollname);
-				if (port == 143 &&  ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need port 993/service imaps for --ssl.\n"), ctl->server.pollname);
-				break;
+			    if (port == 993 && !ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need --ssl for port 993/service imaps.\n"), ctl->server.pollname);
+			    if (port == 143 &&  ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need port 993/service imaps for --ssl.\n"), ctl->server.pollname);
+			    break;
+		    }
 		}
 	    }
 	    if (ctl->listener == LMTP_MODE)
@@ -1484,10 +1502,19 @@ static int load_params(int argc, char **argv, int optind)
 	    run.postmaster = "postmaster";
     }
 
+    if (st) {
+	if (outlevel >= O_DEBUG) {
+	    report(stderr, GT_("cannot parse rcfile, debug mode: dumping obtained configuration as Python:\n"));
+	    puts("raise Exception(\"This output is not meant to be processed by fetchmailrc.\")");
+	    dump_config(&run, querylist);
+	}
+	exit(PS_SYNTAX);
+    }
+
     return(implicitmode);
 }
 
-static RETSIGTYPE terminate_poll(int sig)
+static void terminate_poll(int sig)
 /* to be executed at the end of a poll cycle */
 {
 
@@ -1505,7 +1532,7 @@ static RETSIGTYPE terminate_poll(int sig)
 #endif /* POP3_ENABLE */
 }
 
-static RETSIGTYPE terminate_run(int sig)
+static void terminate_run(int sig)
 /* to be executed on normal or signal-induced termination */
 {
     struct query	*ctl;
@@ -1527,10 +1554,6 @@ static RETSIGTYPE terminate_run(int sig)
 	if (ctl->password)
 	  memset(ctl->password, '\0', strlen(ctl->password));
 
-#if !defined(HAVE_ATEXIT)
-    fm_lock_release();
-#endif
-
     if (activecount == 0)
 	exit(PS_NOMAIL);
     else
@@ -1674,10 +1697,8 @@ static void dump_params (struct runctl *runp,
 	printf(GT_("Logfile is %s\n"), runp->logfile);
     if (strcmp(runp->idfile, IDFILE_NAME))
 	printf(GT_("Idfile is %s\n"), runp->idfile);
-#if defined(HAVE_SYSLOG)
     if (runp->use_syslog)
 	printf(GT_("Progress messages will be logged via syslog\n"));
-#endif
     if (runp->invisible)
 	printf(GT_("Fetchmail will masquerade and will not generate Received\n"));
     if (runp->showdots)
@@ -1780,7 +1801,7 @@ static void dump_params (struct runctl *runp,
 	case A_KERBEROS_V5:
 	    printf(GT_("  Kerberos V5 authentication will be forced.\n"));
 	    break;
-	case A_SSH:
+	case A_IMPLICIT:
 	    printf(GT_("  End-to-end encryption assumed.\n"));
 	    break;
 	}
@@ -1810,9 +1831,14 @@ static void dump_params (struct runctl *runp,
 	if (ctl->server.timeout > 0)
 	    printf(GT_("  Server nonresponse timeout is %d seconds"), ctl->server.timeout);
 	if (ctl->server.timeout ==  CLIENT_TIMEOUT)
-	    printf(GT_(" (default).\n"));
-	else
-	    printf(".\n");
+	    printf(GT_(" (default)"));
+	printf(".\n");
+
+	if (ctl->server.idle_timeout > 0)
+	    printf(GT_("  Server IDLE timeout is %d seconds"), ctl->server.idle_timeout);
+	if (ctl->server.idle_timeout ==  CLIENT_IDLE_TIMEOUT)
+	    printf(GT_(" (default)"));
+	 printf(".\n");
 
 	if (MAILBOX_PROTOCOL(ctl)) 
 	{
@@ -1833,6 +1859,10 @@ static void dump_params (struct runctl *runp,
 	    printf(ctl->keep
 		   ? GT_("  Fetched messages will be kept on the server (--keep on).\n")
 		   : GT_("  Fetched messages will not be kept on the server (--keep off).\n"));
+	    if (ctl->moveto != NULL)
+		printf(GT_("  Message will be moved to %s folder and won't be deleted.\n"), ctl->moveto);
+	    else
+		printf(GT_("  Message will be deleted (no --moveto).\n"));
 	    printf(ctl->flush
 		   ? GT_("  Old messages will be flushed before message retrieval (--flush on).\n")
 		   : GT_("  Old messages will not be flushed before message retrieval (--flush off).\n"));
@@ -1857,6 +1887,9 @@ static void dump_params (struct runctl *runp,
 	    printf(ctl->idle
 		   ? GT_("  Idle after poll is enabled (idle on).\n")
 		   : GT_("  Idle after poll is disabled (idle off).\n"));
+	    printf(ctl->forceidle
+		   ? GT_("  Idle after poll is forced (forceidle on).\n")
+		   : GT_("  Idle after poll is not forced (forceidle off).\n"));
 	    printf(ctl->dropstatus
 		   ? GT_("  Nonempty Status lines will be discarded (dropstatus on)\n")
 		   : GT_("  Nonempty Status lines will be kept (dropstatus off)\n"));
